/**
	\file
	\brief Miscellaneous data types, definitions, functions, &c.

	Set of functions, data types for general use.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_common.h,v 1.2 2010/04/24 19:10:09 dmotl Exp $
*/

#ifndef _CMPACK_COMMON_H_INCLUDED
#define _CMPACK_COMMON_H_INCLUDED

#include "cmpack_common.h"

#ifdef __cplusplus
extern "C" {
#endif

/********************   Public data types   ********************************/

/** \brief Date in full format */
typedef struct _CmpackDate
{
	int year;				/**< year in 4-digit format */
	int month;				/**< month (1-12) */
	int day;				/**< day (1-31) */
} CmpackDate;

/** \brief Time in full format */
typedef struct _CmpackTime
{
	int hour;				/**< hour (0-23) */
	int minute;				/**< minute (0-59) */
	int second;				/**< second (0-59) */
	int milisecond;			/**< milisecond (0-999) */
} CmpackTime;

/** \brief Date and time in full format */
typedef struct _CmpackDateTime
{
	CmpackDate date;		/**< date, see CmpackDate structure */
	CmpackTime time;		/**< time, see CmpackTime structure */
} CmpackDateTime;

/** \brief Border size */
typedef struct _CmpackBorder
{
	int left;					/**< Left border */
	int top;					/**< Top border */
	int right;					/**< Right border */
	int bottom;					/**< Bottom border */
} CmpackBorder;

/** \brief Open flags */
typedef enum _CmpackOpenMode
{
	CMPACK_OPEN_READWRITE	= 0x00,		/**< The file allows modifications */
	CMPACK_OPEN_READONLY	= 0x01,		/**< The file does not allow modifications */
	CMPACK_OPEN_CREATE	    = 0x02,		/**< Create a new file */
	CMPACK_OPEN_HEADONLY	= 0x04		/**< Read only header */
} CmpackOpenMode;

/** \brief Load flags */
typedef enum _CmpackLoadMode
{
	CMPACK_LOAD_DEFAULT		= 0x00,		/**< Load entire file */
	CMPACK_LOAD_HEADONLY	= 0x04		/**< Read only header */
} CmpackLoadMode;

/** \brief Image data types */
typedef enum _CmpackBitpix
{
	CMPACK_BITPIX_UNKNOWN = 0,			/**< Unknown or invalid data format */
	CMPACK_BITPIX_SCHAR = 8,			/**< Signed char (1 bytes) */
	CMPACK_BITPIX_UCHAR = 10,			/**< Unsigned char (1 bytes) */
	CMPACK_BITPIX_SSHORT = 16,			/**< Signed short (2 bytes) */
	CMPACK_BITPIX_USHORT = 20,			/**< Unsigned short (2 bytes) */
	CMPACK_BITPIX_SLONG = 32,			/**< Signed long (4 bytes) */
	CMPACK_BITPIX_ULONG = 40,			/**< Unsigned long (4 bytes) */
	CMPACK_BITPIX_FLOAT = -32,			/**< Single precision FP (4 bytes) */
	CMPACK_BITPIX_DOUBLE = -64			/**< Double precision FP (8 bytes) */
} CmpackBitpix;

#define CMPACK_BITPIX_AUTO CMPACK_BITPIX_UNKNOWN

/** \brief Parameter identifiers */
typedef enum _CmpackParam
{
	CMPACK_PARAM_JD_PRECISION		= 0,	/**< Number of decimal places for Julian dates and heliocentric correction */
	CMPACK_PARAM_POS_PRECISION		= 1,	/**< Number of decimal places for position in pixels */
	CMPACK_PARAM_EXP_PRECISION		= 2,	/**< Number of decimal places for exposure duration in seconds */
	CMPACK_PARAM_TEMP_PRECISION		= 3,	/**< Number of decimal places for CCD temperature */
	CMPACK_PARAM_AMASS_PRECISION	= 4,	/**< Number of decimal places for air-mass coefficient */
	CMPACK_PARAM_ALT_PRECISION		= 5		/**< Number of decimal places for altitude in degrees */
} CmpackParam;

/** \brief Error codes */
typedef enum _CmpackError
{
	CMPACK_ERR_OK				= 0,		/**< Operation finished successfully */
	CMPACK_ERR_MEMORY			= 1001,		/**< Insufficient memory */
	CMPACK_ERR_KEY_NOT_FOUND    = 1002,		/**< Key not found */
	CMPACK_ERR_COL_NOT_FOUND	= 1003,		/**< Column not found */
	CMPACK_ERR_ROW_NOT_FOUND	= 1004,		/**< Row not found */
	CMPACK_ERR_AP_NOT_FOUND		= 1005,		/**< Aperture not found */
	CMPACK_ERR_READ_ONLY		= 1006,		/**< File is open in read-only mode */
	CMPACK_ERR_CLOSED_FILE      = 1007,		/**< Operation not allowed on closed file */
	CMPACK_ERR_OPEN_ERROR		= 1008,		/**< Error while opening file */
	CMPACK_ERR_READ_ERROR		= 1009,		/**< Error while reading file */
	CMPACK_ERR_WRITE_ERROR		= 1010,		/**< Error while writing file */
	CMPACK_ERR_UNKNOWN_FORMAT	= 1011,		/**< Unknown format of source file */
	CMPACK_ERR_BUFFER_TOO_SMALL = 1012,		/**< Buffer is too small */
	CMPACK_ERR_INVALID_CONTEXT  = 1013,     /**< Invalid context */
	CMPACK_ERR_OUT_OF_RANGE     = 1014,     /**< Index is out of range */
	CMPACK_ERR_UNDEF_VALUE      = 1015,     /**< Undefined value */
	CMPACK_ERR_MAG_NOT_FOUND    = 1016,		/**< Measurement not found */
	CMPACK_ERR_STAR_NOT_FOUND	= 1017,		/**< Star not found */
	CMPACK_ERR_NOT_IMPLEMENTED  = 1018,		/**< Unsupported operation */
	CMPACK_ERR_INVALID_SIZE     = 1100,		/**< Invalid dimensions of image */
	CMPACK_ERR_INVALID_DATE     = 1101,		/**< Invalid date/time format */
	CMPACK_ERR_INVALID_PAR      = 1102,		/**< Invalid value of parameter */
	CMPACK_ERR_INVALID_RA		= 1103,		/**< Invalid RA format */
	CMPACK_ERR_INVALID_DEC		= 1104,		/**< Invalid DEC format */
	CMPACK_ERR_INVALID_EXPTIME	= 1105,		/**< Invalid exposure duration */
	CMPACK_ERR_INVALID_BITPIX   = 1106,     /**< Image image data format */
	CMPACK_ERR_INVALID_LON		= 1107,		/**< Invalid format of longitude */
	CMPACK_ERR_INVALID_LAT		= 1108,		/**< Invalid format of latitude */
	CMPACK_ERR_INVALID_JULDAT	= 1109,		/**< Invalid date/time of observation */
	CMPACK_ERR_CANT_OPEN_SRC	= 1200,		/**< Cannot open the source file */
	CMPACK_ERR_CANT_OPEN_OUT	= 1201,		/**< Cannot open the destination file */
	CMPACK_ERR_CANT_OPEN_BIAS	= 1202,		/**< Bias frame not found */
	CMPACK_ERR_CANT_OPEN_DARK	= 1203,		/**< Dark frame not found */
	CMPACK_ERR_CANT_OPEN_FLAT	= 1204,		/**< Flat frame not found */
	CMPACK_ERR_CANT_OPEN_REF	= 1205,		/**< Reference file not found */
	CMPACK_ERR_CANT_OPEN_PHT	= 1206,		/**< Cannot open photometry file */
	CMPACK_ERR_DIFF_SIZE_SRC	= 1300,		/**< Input frames are not compatible (different sizes) */
	CMPACK_ERR_DIFF_SIZE_BIAS   = 1301,		/**< Bias frame is not compatible (different sizes) */
	CMPACK_ERR_DIFF_SIZE_DARK  	= 1302,		/**< Dimensions of dark-frame and scientific image are different */
	CMPACK_ERR_DIFF_SIZE_FLAT   = 1303,		/**< Dimensions of flat-frame and scientific image are different */ 
	CMPACK_ERR_DIFF_BITPIX_SRC	= 1304,		/**< Input frames are not compatible (different image data type) */
	CMPACK_ERR_NO_INPUT_FILES	= 1400,		/**< No input files */
	CMPACK_ERR_NO_BIAS_FRAME	= 1401,		/**< Missing bias frame */
	CMPACK_ERR_NO_DARK_FRAME	= 1402,		/**< Missing dark frame */
	CMPACK_ERR_NO_FLAT_FRAME	= 1403,		/**< Missing flat frame */
	CMPACK_ERR_NO_OBS_COORDS	= 1404,		/**< Missing observer's coordinates */
	CMPACK_ERR_NO_OBJ_COORDS	= 1405,		/**< Missing object's coordinates */
	CMPACK_ERR_NO_OUTPUT_FILE	= 1406,		/**< Missing name of output frame */
	CMPACK_ERR_NO_REF_FILE		= 1407,		/**< Missing name of reference frame */
	CMPACK_ERR_MEAN_ZERO    	= 1500,		/**< Mean value of flat frame is zero (can't divide by zero) */
	CMPACK_ERR_REF_NOT_FOUND	= 1501,		/**< Refererence star was not found */
	CMPACK_ERR_FEW_POINTS_REF	= 1502,		/**< Too few stars in the reference file */
	CMPACK_ERR_FEW_POINTS_SRC	= 1503,		/**< Too few stars in the source file */
	CMPACK_ERR_MATCH_NOT_FOUND	= 1504		/**< Coincidences not found  */
} CmpackError;

/** \brief Mask flags (tells which fields are valid) */
typedef enum _CmpackMask
{
	CMPACK_MASK_CODE		= 0x000001,		/**< Result code */
	CMPACK_MASK_MAG     	= 0x000002,		/**< Magnitude */
	CMPACK_MASK_ERR     	= 0x000004,		/**< Error */
	CMPACK_MASK_POS			= 0x000008,		/**< Position */
	CMPACK_MASK_RA   		= 0x000010,		/**< Right ascension */
	CMPACK_MASK_DEC     	= 0x000020,		/**< Declination */
	CMPACK_MASK_LAT     	= 0x000040,		/**< Latitude */
	CMPACK_MASK_LON			= 0x000080,		/**< Longitude */
	CMPACK_MASK_ID			= 0x000100,		/**< Identifier */
	CMPACK_MASK_SEL_TYPE	= 0x000800,		/**< Selection type */
	CMPACK_MASK_SEL_INDEX	= 0x001000,		/**< Selection index */
	CMPACK_MASK_IMG_SIZE	= 0x002000,		/**< Image width */
	CMPACK_MASK_JD			= 0x008000,		/**< Julian date of observation */
	CMPACK_MASK_EXP_TIME	= 0x010000,		/**< Exposure duration */
	CMPACK_MASK_CCD_TEMP	= 0x020000,		/**< CCD temperature */
	CMPACK_MASK_FILTER      = 0x040000,		/**< Color filter designation */
	CMPACK_MASK_OBJ_NAME	= 0x080000,		/**< Object's designation */
	CMPACK_MASK_RADIUS      = 0x100000,		/**< Radius */
	CMPACK_MASK_OBS_NAME    = 0x200000		/**< Observer's name */
} CmpackMask;

/********************   Intialization/clean-up   ********************************/

/**
	\brief Initialize library
	\details Call this function in the main thread before any other function from 
	the C-Munipack library is called.
	\warning Any call to a function from the C-Munipack library preceding execution
	of this function can lead to unpredictable results!
*/
void cmpack_init(void);

/**
	\brief Clean-up library
	\details Call this function in the main thread as the last of all functions from
	the C-Munipack library. When compiled with _DEBUG flag on, prints table of unreleased 
	memory blocks to stderr.
	\warning Any call to a function from the C-Munipack library succeeding execution
	of this function can lead to unpredictable results!
*/	
void cmpack_cleanup(void);

/********************   Error handling   ********************************/

/**
	\brief Translates error code to its human readable representation.
	\details The function returns pointer to allocated memory buffer. The
	caller is responsible to release it by calling cmpack_free() function.
	\param[in] code			error code
	\return pointer to allocated string or zero on failure.
*/
char *cmpack_formaterror(int code);

/********************   Version information   ********************************/

/**
	\brief Get C-Munipack library version
	\details The version identifier consists of three part. The first one
	is major number which is incremented on major changes in the library.
	The second one is minor version incremented on changes in the library's
	interface. Even minor numbers indicates stable versions, odd minor numbers
	are reserved for development versions. The last part, the revision number,
	is incremented every time the package(s) are published.

	\warning The function returns pointer to internal memory buffer. Do not call
	cmpack_free() on the result!

	\return pointer to allocated string or zero on failure.
*/
const char *cmpack_versionid(void);

/**
	\brief Get major version number
	\details This part is incremented on major changes in the library.
	\return major version number
*/
int cmpack_major_version(void);

/**
	\brief Get minor version number
	\details This part is incremented on changes in the library's
	interface. Even minor numbers indicates stable versions, odd minor numbers
	are reserved for development versions. 
	\return minor version number
*/
int cmpack_minor_version(void);

/**
	\brief Get revision number
	\details This part is incremented every time the package(s) are published.
	\return revision number
*/
int cmpack_revision_number(void);

/**
	\brief Get C-Munipack package name

	\warning The function returns pointer to internal memory buffer. Do not call
	cmpack_free() on the result!

	\return pointer to allocated string or zero on failure.
*/
const char *cmpack_packagename(void);

/********************   Common functions   ********************************/

/**
	\brief Fill date structure
	\param[out] date		date structure
	\param[in] year			year in 4-digit format
	\param[in] month		month (1-12)
	\param[in] day			day (1-31)
*/
void cmpack_date_set(CmpackDate *date, int year, int month, int day);

/**
	\brief Fill time structure
	\param[out] time		time structure
	\param[in] hour			hour (0-23)
	\param[in] minute		minute (0-59)
	\param[in] second		second (0-59)
	\param[in] milisecond	milisecond (0-999)
*/
void cmpack_time_set(CmpackTime *time, int hour, int minute, int second, int milisecond);

/**  
	\brief Fill date and time structure
	\param[out] datetime	date and time structure
	\param[in] year			year in 4-digit format
	\param[in] month		month (1-12)
	\param[in] day			day (1-31)
	\param[in] hour			hour (0-23)
	\param[in] minute		minute (0-59)
	\param[in] second		second (0-59)
	\param[in] milisecond	milisecond (0-999)
*/
void cmpack_datetime_set(CmpackDateTime *datetime, int year, int month, int day,
						 int hour, int minute, int second, int milisecond);

/**
	\brief Fill border structure
	\param[out] border		border structure
	\param[in] left			left border 
	\param[in] top			top border 
	\param[in] right		right border 
	\param[in] bottom		bottom border 
*/
void cmpack_border_set(CmpackBorder *border, int left, int top, int right, int bottom);

/**
	\brief Get configuration parameter
	\param[in] param		parameter identifier (see CMPACK_PARAM_xxx)
	\return parameter value or zero on failure 
*/
long cmpack_get_param_long(CmpackParam param);

/******************   Memory management   ***************************/

/**
	\brief Allocate memory.
	\details The function allocates specified amount of memory on heap
	and returns pointer to the beginning of the memory block. All
	blocks allocated by means of this function must be released by
	calling the cmpack_free() function.
	The content of the memory block is undefined.
	\param[in] size				size in bytes
	\return pointer to the first byte of the memory block
*/
void *cmpack_malloc(size_t size);

/**
	\brief Allocate and clear memory block.
	\details The function allocates specified amount of memory on heap
	and returns pointer to the beginning of the memory block and sets
	the content to zeros. All blocks allocated by means of this function 
	must be released by	calling the cmpack_free() function.
	\param[in] num				number of items
	\param[in] size				size of single item
	\return pointer to the first byte of the memory block
*/
void *cmpack_calloc(size_t num, size_t size);

/**
	\brief Change size of the memory block or make new one.
	\details When pointer to the existing memory block is NULL, it allocates
	new memory block of specified size. Otherwise, it resizes the memory block
	to specified size. All content of the old memory is preserved, but the 
	content of new area is undefined.
	\param[in] ptr				pointer to existing block (or NULL)
	\param[in] size				new size in bytes
*/
void *cmpack_realloc(void *ptr, size_t size);

/**
	\brief Makes copy of a null-terminated string
	\details Returns pointer to newly allocated memory block. Use cmpack_free()
	function to release it. If the input string is NULL, it returns NULL. If
	the input string is empty, it returns empty string.
	\param[in] src				source string
*/
char *cmpack_strdup(const char *src);

/**
	\brief Updates a null-terminated string
	\details First parameter can specify a pointer to an existing null-terminated
	string. If it's non-NULL, it's realocated to fit a new string and makes a copy
	of a string. Otherwise it allocates a new memory buffer and make a copy of a 
	string. Old pointer may not be valid after that function and should not be used
	in following code, always use the result of the function.
	\param[in] dst				pointer to existing block (or NULL)
	\param[in] src				source string
*/
char *cmpack_strset(char *dst, const char *src);

/**
	\brief Release allocated memory
	\details Use this function to release the memory block allocated by the
	cmpack_malloc(), cmpack_malloc_dbg(), cmpack_calloc(), cmpack_calloc_dbg(),
	cmpack_realloc(), cmpack_realloc_dbg() functions or a other functions
	from the C-Munipack library.
	\param[in] ptr				pointer to memory
*/
void cmpack_free(void *ptr);

/******************   Debug heap functions   ******************************/

/**
	\brief Allocate memory (debug version)
	\details The function allocates specified amount of memory on heap
	and returns pointer to the beginning of the memory block. All
	blocks allocated by means of this function must be released by
	calling the cmpack_free() function. Unlike cmpack_malloc(), this
	function stores the given name of the source file and line number.
	The content of the memory block is undefined.
	\param[in] size				size in bytes
	\param[in] file				source file name
	\param[in] line				source line index
	\returns pointer to the first byte of the memory block
*/
void *cmpack_malloc_dbg(size_t size, const char *file, int line);

/**
	\brief Change size of the memory block or make new one.
	\details When pointer to the existing memory block is NULL, it allocates
	new memory block of specified size. Otherwise, it resizes the memory block
	to specified size. All content of the old memory is preserved, but the 
	content of new area is undefined. Unline cmpack_realloc() this function 
	stores the given name of the source file and line number.
	\param[in] ptr				pointer to existing block (or NULL)
	\param[in] size				new size in bytes
	\param[in] file				source file name
	\param[in] line				source line index
*/
void *cmpack_realloc_dbg(void *ptr, size_t size, const char *file, int line);

/**
	\brief Allocate and clear memory block (debug version)
	\details The function allocates specified amount of memory on heap
	and returns pointer to the beginning of the memory block and sets
	the content to zeros. All blocks allocated by means of this function 
	must be released by	calling the cmpack_free() function. Unlike 
	cmpack_calloc(), this function stores the given name of the source 
	file and line number.
	\param[in] num				number of items
	\param[in] size				size of single item
	\param[in] file				source file name
	\param[in] line				source line index
	\return pointer to the first byte of the memory block
*/
void *cmpack_calloc_dbg(size_t num, size_t size, const char *file, int line);

/**
	\brief Makes copy of a null-terminated string
	\details Returns pointer to newly allocated memory block. Use cmpack_free()
	function to release it. If the input string is NULL, it returns NULL. If
	the input string is empty, it returns empty string. Unline cmpack_realloc() 
	this function stores the given name of the source file and line number.
	\param[in] src				source string
	\param[in] file				source file name
	\param[in] line				source line index
*/
char *cmpack_strdup_dbg(const char *src, const char *file, int line);

/**
	\brief Updates a null-terminated string
	\details First parameter can specify a pointer to an existing null-terminated
	string. If it's non-NULL, it's realocated to fit a new string and makes a copy
	of a string. Otherwise it allocates a new memory buffer and make a copy of a 
	string. Old pointer may not be valid after that function and should not be used
	in following code, always use the result of the function.
	\param[in] dst				pointer to existing block (or NULL)
	\param[in] src				source string
	\param[in] file				source file name
	\param[in] line				source line index
*/
char *cmpack_strset_dbg(char *dst, const char *src, const char *file, int line);

#ifdef _DEBUG
#ifndef CMPACK_MEM_DO_NOT_USE_DEFINES
#define cmpack_malloc(size)			cmpack_malloc_dbg((size), __FILE__, __LINE__)
#define cmpack_calloc(num, size)	cmpack_calloc_dbg((num), (size), __FILE__, __LINE__)
#define cmpack_realloc(ptr, size)	cmpack_realloc_dbg((ptr), (size), __FILE__, __LINE__)
#define cmpack_strdup(src)			cmpack_strdup_dbg((src), __FILE__, __LINE__)
#define cmpack_strset(dst, src)		cmpack_strset_dbg((dst), (src), __FILE__, __LINE__)
#endif
#endif

/******************   Reference counting   ***************************/

/**
	\brief Increment reference counter
	\details Call this function to make a persistent reference on a context. 
	It ensures, that it won't be released until you call cmpack_unref() on it.
	\param[in] ctx			any context
	\return the same value as ctx
*/
void *cmpack_addref(void *ctx); 

/**
	\brief Decrement reference counter and when it is zero free the 
	allocated memory
	\details Call this function on a context to release your
	persistent reference made by cmpack_addref() function.
	\param[in] ctx			any context
*/
void cmpack_unref(void *ctx);

/******************** Date & time conversion functions ********************/

/**
	\brief Parse string and return parts of the date
	\param[in] datestr		date string (YYYY-MM-DD)
	\param[out] date		date parts
	\return zero on success or error code on failure
*/
int cmpack_strtodate(const char *datestr, CmpackDate *date);

/**
	\brief Parse string and return parts of the time
	\param[in] timestr		time string (HH:MM:SS.SSS)
	\param[out] time		time parts
	\return zero on success or error code on failure
*/
int cmpack_strtotime(const char *timestr, CmpackTime *time);

/**
	\brief Print date to the string
	\param[in] date			date parts
	\param[out] buf			output string
	\param[in] buflen		size of buffer in chars
	\return zero on success or error code on failure.
*/
int cmpack_datetostr(const CmpackDate *date, char *buf, int buflen);

/**
	\brief Print time to the string
	\param[in] time			time parts
	\param[out] buf			output string
	\param[in] buflen		size of buffer in chars
	\return zero on success or error code on failure.
*/
int cmpack_timetostr(const CmpackTime *time, char *buf, int buflen);

/**
	\brief Convert date and time parts to julian date
	\param[in] datetime		date and time 
	\return julian date on success or zero on failure 
*/
double cmpack_encodejd(const CmpackDateTime *datetime);

/**
	\brief Convert julian date to parts of date and time
	\param[in] jd			julian date
	\param[out] datetime	date and time
	\return zero on success or error code on failure.
*/
int cmpack_decodejd(double jd, CmpackDateTime *datetime);

/******************** Conversion of coordinates **********************/

/**
	\brief Convert the string to R.A.
	\param[in] buf			input string 
	\param[out] ra			right ascension in hours (0..24)
	\return zero on success or error code on failure.
*/
int  cmpack_strtora(const char *buf, double *ra);

/**
	\brief Converts the string to DEC.
	\param[in] buf			input string
	\param[out] dec			declination in degrees (-90..90)
	\return zero on success or error code on failure.
*/
int  cmpack_strtodec(const char *buf, double *dec);

/**
	\brief Converts the R.A. value to the string (HH MM SS)
	\param[in] ra			right ascension in hours
	\param[out] buf			output string
	\param[in] buflen		size of buffer in chars
	\return zero on success or error code on failure.
*/
int cmpack_ratostr(double ra, char *buf, int buflen);

/**
	\brief Converts the DEC. value to the string (+DDD MM SS)
	\param[in] dec			declination in degrees
	\param[out] buf			output string
	\param[in] buflen		size of buffer in chars
	\return pointer to allocated string or zero on failure
*/
int cmpack_dectostr(double dec, char *buf, int buflen);

/**
	\brief Converts string to latitude.
	\param[in] buf			input string
	\param[out] lat			latitude in degrees (-90..90)
	\return zero on success or error code on failure.
*/
int cmpack_strtolat(const char *buf, double *lat);

/**
	\brief Converts string to longitude.
	\param[in] buf			input string
	\param[out] lon			longitude in degrees (-180..180)
	\return zero on success or error code on failure.
*/
int cmpack_strtolon(const char *buf, double *lon);

/**
	\brief Converts the latitude to the string (N DDD MM SS)
	\details The function returns pointer to allocated memory buffer. The
	caller is responsible to release it by calling cmpack_free() function.
	\param[in] lat			latitude in degrees
	\param[out] buf			output string
	\param[in] buflen		size of buffer in chars
	\return zero on success or error code on failure.
*/
int cmpack_lattostr(double lat, char *buf, int buflen);

/**
	\brief Converts the longitude to the string (E DDD MM SS)
	\details The function returns pointer to allocated memory buffer. The
	caller is responsible to release it by calling cmpack_free() function.
	\param[in] lon			longitude in degrees
	\param[out] buf			output string
	\param[in] buflen		size of buffer in chars
	\return zero on success or error code on failure.
*/
int cmpack_lontostr(double lon, char *buf, int buflen);

/******************** Airmass coefficient **********************/

/**
	\brief Computes Airmass coefficient
	\param[in] jd			julian date (UT)
	\param[in] obj_ra		right ascension of object in hours
	\param[in] obj_dec		declination of object in degrees
	\param[in] obs_lon		longitude of observer in degrees
	\param[in] obs_lat		latitude of observer in degrees
	\param[out] alt			altitude in degrees (optional)
	\return airmass coefficient (X=1 for object in zenith)
*/
double cmpack_airmass(double jd, double obj_ra, double obj_dec, double obs_lon, 
					  double obs_lat, double *alt);

/******************** Heliocentric correction **********************/

/**
	\brief Computes heliocentric correction
	\param[in] jd			julian date (UT)
	\param[in] obj_ra		right ascension of object in hours
	\param[in] obj_dec		declination of object in degrees
	\return heliocentric correction in days
*/
double cmpack_helcorr(double jd, double obj_ra, double obj_dec);

/************************ Robust mean functions **********************************/

/**
	\brief Computes robust mean and standard deviation
	\details This function is slower but more precise that cmpack_fastmean().
	\param[in] n			number of elements
	\param[in] A			array of elements
	\param[out] mean		robust mean
	\param[out] sig			standard deviation
*/
int cmpack_robustmean(int n, double *A, double *mean, double *sig);

/**
	\brief Computes estimation of robust mean and standard deviation
	\details This function is much faster but less precise that cmpack_robustmean().
	Unlike cmpack_robustmean(), this function estimates the mean value
	and standard deviation by computing robust mean using a subset of original
	data. At most \a nmax samples are taken in and all values <=lobad or >= hibad
	are ruled out. 
	\param[in] n			number of elements
	\param[in] A			array of elements
	\param[in] nmax			max. number of samples in a subset
	\param[in] lobad		low cutoff for sample value
	\param[in] hibad		high cutoff for sample value
	\param[out] skymed		robust mean
	\param[out] skysig		standard deviation
*/
int cmpack_fastmean(int n, double *A, int nmax, double lobad, double hibad, double *skymed, double *skysig);

#ifdef __cplusplus
}
#endif

#endif
