/**
	\file
	\brief Functions for the heliocentric correction

	Set of functions defined in this module allows user to 
	apply heliocentric correction to light-curve data files.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_hcorr.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/
#ifndef _CMPACK_HCORR_H_INCLUDED
#define _CMPACK_HCORR_H_INCLUDED

#include "cmpack_console.h"
#include "cmpack_table.h"

/********************   Private data structures   ********************************/

/**
	\brief Heliocentric correction context
	\details This private data structure holds the parameter for heliocentric
	correction.
*/
typedef struct _CmpackHelCorr CmpackHelCorr;

/**********************   Public constants   ***********************************/

/**
	\brief Correction flags
*/
typedef enum _CmpackHCorrFlags
{
	CMPACK_HCORR_DEFAULT = 0,			/**< Default behavior */
	CMPACK_HCORR_REVERSE =	(1<<0),		/**< JD is heliocentric, compute geocentric JD */
	CMPACK_HCORR_ADDVALUE =	(1<<1),		/**< Add value of hel. cor. to a new column */
	CMPACK_HCORR_NOMODIFY =	(1<<2)		/**< Do not modify values of JD */
} CmpackHCorrFlags;

/********************   Public functions   *************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new heliocentric correction context.
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to context or zero on failure
*/
CmpackHelCorr *cmpack_hcorr_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			air-mass computation context
	\param[in] con			console context
*/
void cmpack_hcorr_set_console(CmpackHelCorr *ctx, CmpackConsole *con);

/**
	\brief Set object's right ascension
	\param[in] ctx			heliocentric correction context
	\param[in] ra			right ascension in hours
*/
void cmpack_hcorr_set_ra(CmpackHelCorr *ctx, double ra);

/**
	\brief Get object's right ascension
	\param[in] ctx			heliocentric correction context
	\param[out] ra			right ascension in hours
	\return zero on success or error code on failure
*/
int cmpack_hcorr_get_ra(CmpackHelCorr *ctx, double *ra);

/**
	\brief Set object's declination
	\param[in] ctx			heliocentric correction context
	\param[in] dec			declination in degrees
*/
void cmpack_hcorr_set_dec(CmpackHelCorr *ctx, double dec);

/**
	\brief Get object's declination
	\param[in] ctx			heliocentric correction context
	\param[out] dec			declination in degrees
	\return zero on success or error code on failure
*/
int cmpack_hcorr_get_dec(CmpackHelCorr *ctx, double *dec);

/**
	\brief Make heliocentric correction of data stored in a table.
	\param[in] ctx			correction context
	\param[in] table		table
	\param[in] column		column containing values of JD
	\param[in] flags		see CMPACK_HCORR_xxx constants
	\return zero on success or error code on failure.
*/
int cmpack_hcorr(CmpackHelCorr *ctx, CmpackTable *table, int column, CmpackHCorrFlags flags);

/**
	\brief Compute heliocentric correction for single julian date.
	\param[in] ctx			heliocentric correction context
	\param[in] jd			julian date of observation
	\param[out] helcor		heliocentric correction in days
	\return zero on success or error code on failure
*/
int cmpack_hcorr_jd(CmpackHelCorr *ctx, double jd, double *helcor);

#ifdef __cplusplus
}
#endif

#endif
