/**
	\file
	\brief Functions for the making master-bias correction frame

	Set of functions defined in this module allows user to 
	make a master-bias frame from a set of CCD frames.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_mbias.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/
#ifndef _CMPACK_MBIAS_H_INCLUDED
#define _CMPACK_MBIAS_H_INCLUDED

#include "cmpack_common.h"
#include "cmpack_console.h"
#include "cmpack_ccdfile.h"

/********************   Private data structures   ********************************/

/**
	\brief Configuration context for the Master-bias tool
	\details This private data structure holds the configuration parameter 
	for the master-bias tool.
*/
typedef struct _CmpackMasterBias CmpackMasterBias;

/********************   Public functions   ********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new context for the Master-bias tool
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to the new master-bias context or zero on failure
*/
CmpackMasterBias *cmpack_mbias_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			master-bias context
	\param[in] con			console context
*/
void cmpack_mbias_set_console(CmpackMasterBias *ctx, CmpackConsole *con);

/**
	\brief Set output data format
	\param[in] ctx			master-bias context
	\param[in] bitpix		output data format
*/
void cmpack_mbias_set_bitpix(CmpackMasterBias *ctx, CmpackBitpix bitpix);

/**
	\brief Get output data format
	\param[in] ctx			master-bias context
	\return output data format
*/
CmpackBitpix cmpack_mbias_get_bitpix(CmpackMasterBias *ctx);

/**
	\brief Set image border size
	\details If you set the border to nonzero size, the conversion function
	will set the pixels which belongs to the border area to zero. You can use
	this feature to clear an unusable part of a frame.
	\param[in] ctx			master-bias context
	\param[in] border		border size in pixels
*/
void cmpack_mbias_set_border(CmpackMasterBias *ctx, const CmpackBorder *border);

/**
	\brief Get image flip flags
	\param[in] ctx			master-bias context
	\param[out] border		border size in pixels
*/
void cmpack_mbias_get_border(CmpackMasterBias *ctx, CmpackBorder *border);

/**
	\brief Open output file
	\details Opens new master-bias frame and returns its context structure.
	The caller is responsible to close the file and free the allocated
	memory by calling the cmpack_mbias_close() function.
	\param[in] ctx			master-bias context
	\param[in] outfile		output file context
	\return zero on success or error code on failure.
*/
int cmpack_mbias_open(CmpackMasterBias *ctx, CmpackCcdFile *outfile);

/**
	\brief Add a frame to the accumulation buffer
	\details The function reads the frame from specified CCD-frame 
	file into memory.
	\param[in] ctx			master-bias context
	\param[in] infile		input file context
	\return zero on success or error code on failure.
*/
int cmpack_mbias_read(CmpackMasterBias *ctx, CmpackCcdFile *infile);

/**
	\brief Make master-bias frame and save it to the file
	\details The function computes the output CCD data, writes it
	to the file, closes the output file and frees allocated data.
	\param[in] ctx			master-bias context
	\return zero on success or error code on failure.
*/
int cmpack_mbias_close(CmpackMasterBias *ctx);

#ifdef __cplusplus
}
#endif

#endif

