/**
	\file
	\brief Functions for the time correction

	Set of functions defined in this module allows user to 
	apply time correction to CCD frames.

	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_tcorr.h,v 1.2 2010/04/24 19:10:09 dmotl Exp $
*/

#ifndef _CMPACK_TCORR_H_INCLUDED
#define _CMPACK_TCORR_H_INCLUDED

#include "cmpack_common.h"
#include "cmpack_console.h"
#include "cmpack_ccdfile.h"

/********************   Private data structures   ********************************/

/**
	\brief Bias-frame correction context
	\details This private data structure holds the parameter for time
	correction as well as the time itself.
*/
typedef struct _CmpackTimeCorr CmpackTimeCorr;

/************************   Public functions   ***********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new time correction context
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to the new time correction context or zero on failure
*/
CmpackTimeCorr *cmpack_tcorr_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			time correction context
	\param[in] con			console context
*/
void cmpack_tcorr_set_console(CmpackTimeCorr *ctx, CmpackConsole *con);

/**
	\brief Set time correction
	\param[in] ctx			time correction context
	\param[in] corr			correction in seconds
*/
void cmpack_tcorr_set_corr(CmpackTimeCorr *ctx, double corr);

/**
	\brief Get time correction
	\param[in] ctx			time correction context
	\return current value of time correction in seconds
*/
double cmpack_tcorr_get_corr(CmpackTimeCorr *ctx);

/**
	\brief Execute time correction on a file
	\details The function reads header data from the file context, performs
	the time correction and stores the output to the same context.
	\param[in] ctx			time correction context
	\param[in,out] file		frame context
	\return zero on success or error code on failure
*/
int cmpack_tcorr(CmpackTimeCorr *ctx, CmpackCcdFile *file);

/**
	\brief Execute time correction on a file
	\details The function reads header data from the infile context, performs
	the time correction and stores the output image to the outfile context.
	\param[in] ctx			time correction context
	\param[in] infile		input frame context
	\param[in] outfile		output frame context
	\return zero on success or error code on failure
*/
int cmpack_tcorr_ex(CmpackTimeCorr *ctx, CmpackCcdFile *infile, CmpackCcdFile *outfile);

#ifdef __cplusplus
}
#endif

#endif
